/*
 * Programming activity
 *
 * Goal: Finish a class hierarchy for filled circles and rectangles.
 * 
 * --------------------------------------------------------------------------------------------------- 
 * Part 1, complete the missing code in Shape.java and Circle.java.
 * Sample console output for part 1:
 * 
 * Circle @ (0.50, 0.50), radius = 0.10, area 0.0314, distance from origin 0.7071
 * Circle @ (0.30, 0.30), radius = 0.20, area 0.1257, distance from origin 0.4243
 * Circle @ (0.70, 0.80), radius = 0.05, area 0.0079, distance from origin 1.0630
 * 
 * distance from Circle @ (0.50, 0.50), radius = 0.10 to Circle @ (0.30, 0.30), radius = 0.20, 0.2828
 * distance from Circle @ (0.50, 0.50), radius = 0.10 to Circle @ (0.70, 0.80), radius = 0.05, 0.3606
 * distance from Circle @ (0.30, 0.30), radius = 0.20 to Circle @ (0.70, 0.80), radius = 0.05, 0.6403
 * 
 * ---------------------------------------------------------------------------------------------------
 * Part 2, using Cirlce.java as a guide, create a new subclass for rectangles.
 * Sample console output for part 2:
 * 
 * Circle @ (0.50, 0.50), radius = 0.10, area 0.0314, distance from origin 0.7071
 * Circle @ (0.30, 0.30), radius = 0.20, area 0.1257, distance from origin 0.4243
 * Circle @ (0.70, 0.80), radius = 0.05, area 0.0079, distance from origin 1.0630
 * Rectangle @ (0.65, 0.15) [0.30 x 0.20], area 0.0600, distance from origin 0.6671
 * 
 * distance from Circle @ (0.50, 0.50), radius = 0.10 to Circle @ (0.30, 0.30), radius = 0.20, 0.2828
 * distance from Circle @ (0.50, 0.50), radius = 0.10 to Circle @ (0.70, 0.80), radius = 0.05, 0.3606
 * distance from Circle @ (0.50, 0.50), radius = 0.10 to Rectangle @ (0.65, 0.15) [0.30 x 0.20], 0.3808
 * distance from Circle @ (0.30, 0.30), radius = 0.20 to Circle @ (0.70, 0.80), radius = 0.05, 0.6403
 * distance from Circle @ (0.30, 0.30), radius = 0.20 to Rectangle @ (0.65, 0.15) [0.30 x 0.20], 0.3808
 * distance from Circle @ (0.70, 0.80), radius = 0.05 to Rectangle @ (0.65, 0.15) [0.30 x 0.20], 0.6519
 */

import java.awt.Color;
import java.util.ArrayList;

public abstract class Shape
{
    private double    x;             // X location of the shape's center
    private double    y;             // Y location of the shape's center
    private Color     color;         // Color of the shape
    
    // Constructor, create at the given (x, y) with the color (r, g, b)
    public Shape(double x, 
                 double y, 
                 double r, 
                 double g, 
                 double b)
    {
        this.x = x;
        this.y = y;
        color = new Color((float) r, (float) g, (float) b);
    }
    
    // Default constructor
    public Shape()
    {
    }
 
    // Getter for the x center of the shape
    public double getX()
    {
        return x;
    }
    
    // Getter for the y center of the shape
    public double getY()
    {
        return y;
    }

    // Getter for the color of the shape
    public Color getColor()
    {
        return color;
    }

    // Calculate distance between the center of this shape an some other shape
    public double distance(Shape other)
    {
        return distance(other.x, other.y);
    }
    
    // Calculate distance from center of this shape to some other (x, y) point
    public double distance(double x, double y)
    {
        double deltaX = this.x - x;
        double deltaY = this.y - y;
        return Math.sqrt(deltaX * deltaX + deltaY * deltaY);
    }
    
    // Calculate the area of this shape
    public abstract double getArea();
    
    // Draw this shape using StdDraw
    public abstract void draw();
    
    // Return a string representation of this shape, namely its location
    public String toString()
    {
        return String.format("(%.2f, %.2f)", x, y);
    }
    
    // Test main program
    public static void main(String [] args)
    {
        ArrayList<Shape> shapes = new ArrayList<Shape>();
        
        // Part 1: add 3 circles to the list of shapes
        shapes.add(new Circle(0.5, 0.5, 1.0, 0.0, 0.0, 0.1));
        shapes.add(new Circle(0.3, 0.3, 0.0, 1.0, 0.0, 0.2));
        shapes.add(new Circle(0.7, 0.8, 0.0, 0.0, 1.0, 0.05));
            
        // Part 2: uncomment the following line to add a rectangle
        shapes.add(new Rectangle(0.65, 0.15, 0.3, 0.0, 0.5, 0.3, 0.2));
        
        // Part 3: bonus level: circles with holes in them
        //shapes.add(new Donut(0.2, 0.8, 0.5, 0.5, 0.5, 0.2, 0.1));
        
        // Part 4: bonus bonus level: donuts with sprinkles!
        //shapes.add(new SprinkledDonut(0.5, 0.5, 0.5, 0.8, 0.9, 0.3, 0.15, 5000));
        
        // Loop over each shape, drawing it and printing out its area
        for (Shape s : shapes)
        {
            s.draw();
            System.out.printf("%s, area %.4f, distance from origin %.4f\n", s, s.getArea(), s.distance(0.0, 0.0));
        }
        System.out.println();
        
        // Compute the distance between the center of all pairs of shapes
        for (int i = 0; i < shapes.size(); i++)
        {
            for (int j = i + 1; j < shapes.size(); j++)
            {
                System.out.printf("distance from %s to %s, %.4f\n", 
                                    shapes.get(i), 
                                    shapes.get(j), 
                                    shapes.get(i).distance(shapes.get(j)));
            }
        }
    }
    
}
